SystemOrganization addCategory: #'Phidget-Development'!

!SketchMorph methodsFor: '*Phidget-Development' stamp: 'pt 9/3/2009 14:45'!
compileFormGeneratorIn: aClass
	"Adds (or modifies) the method #backgroundForm to the class-side of the given class. It returns the form of this SketchMorph"

	self form generateFormCreatingMethodIn: aClass.! !

Object subclass: #PhdDev
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Phidget-Development'!

!PhdDev class methodsFor: 'as yet unclassified' stamp: 'pt 9/3/2009 15:11'!
compileMethodForBackground: aForm  into: aClass
	"To build a #backgroundForm Method in the class side of a class which returns the form of a SketchMorph, execute 'self compileFormGeneratorIn: yourClass' in the SketchMorphs context.
	The same goes for a form if you execute 'generateFormCreatingMethodIn: yourClass'."

	aClass class compile:
		(String streamContents: [:stream |
			stream nextPutAll: 'backgroundForm'; cr; tab; nextPutAll: '^ '.
			aForm storeOn: stream
		]).! !

Object subclass: #StatisticRecord
	instanceVariableNames: 'title packages classes methods linesOfCode'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Phidget-Development'!

!Form methodsFor: '*Phidget-Development' stamp: 'pt 9/3/2009 15:08'!
generateFormCreatingMethodIn: aClass
	"Adds (or modifies) the method #backgroundForm to the class-side of the given class. It returns this form."

	PhdDev compileMethodForBackground: self into: aClass.! !

InfiniteForm subclass: #WorldAdaptingForm
	instanceVariableNames: 'originalForm scalePoint rotationStyle'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Phidget-Development'!

!WorldAdaptingForm class methodsFor: 'as yet unclassified' stamp: 'pt 8/29/2009 20:32'!
setBackgroundTo: aForm

	| world bg |
	world := self currentWorld.
	bg := self new.
	bg
		originalForm: aForm;
		extent: world extent.
	world color: bg.
	world changed.
	
	! !

!WorldAdaptingForm methodsFor: 'as yet unclassified' stamp: 'pt 8/15/2009 08:27'!
extent: newExtent
	"Change my scale to fit myself into the given extent.
	Avoid extents where X or Y is zero."
	(newExtent y = 0 or: [ newExtent x = 0 ]) ifTrue: [ ^self ].
	self extent = newExtent ifTrue:[^self].
	scalePoint := newExtent asFloatPoint / (originalForm extent max: 1@1).
	self layoutChanged.
! !

!WorldAdaptingForm methodsFor: 'as yet unclassified' stamp: 'pt 8/15/2009 08:42'!
generateInterpolatedForm
	"Draw the given form onto the canvas using the Balloon 3D engine"
	| aCanvas extent |
	extent := (originalForm extent * scalePoint) asIntegerPoint.
	self form: (Form extent: extent asIntegerPoint depth: originalForm depth).
	aCanvas := self form getCanvas.
	^self drawInterpolatedImage: originalForm on: aCanvas! !

!WorldAdaptingForm methodsFor: 'as yet unclassified' stamp: 'pt 8/15/2009 08:38'!
generateRotatedForm
	"Compute my rotatedForm and offsetWhenRotated."

	| scalePt smoothPix pair |
	scalePoint ifNil: [scalePoint := 1 @ 1].
	scalePt := scalePoint x abs @ scalePoint y abs.
	rotationStyle == #none ifTrue: [scalePt := 1 @ 1].
	smoothPix := (scalePt x < 1.0 or: [scalePt y < 1.0]) 
		ifTrue: [2]
		ifFalse: [1].
	rotationStyle = #leftRight 
		ifTrue: 
			[self heading asSmallAngleDegrees < 0.0 
				ifTrue: [scalePt := scalePt x negated @ scalePt y]].
	rotationStyle = #upDown 
		ifTrue: 
			[self heading asSmallAngleDegrees abs > 90.0 
				ifTrue: [scalePt := scalePt x @ scalePt y negated]].
	self form: (scalePt = (1 @ 1) 
				ifTrue: [originalForm]
				ifFalse: 
					["ar 11/19/2001: I am uncertain what happens in the case of rotationStyle ~~ normal"

					(rotationStyle == #normal and: [self useInterpolation]) 
						ifTrue: [^self generateInterpolatedForm].
					pair := WarpBlt current 
								rotate: originalForm
								degrees: 0
								center: originalForm boundingBox center
								scaleBy: scalePt
								smoothing: smoothPix.
					pair first])! !

!WorldAdaptingForm methodsFor: 'as yet unclassified' stamp: 'pt 8/15/2009 08:47'!
layoutChanged
	"Update rotatedForm."
	self generateRotatedForm.! !

!WorldAdaptingForm methodsFor: 'as yet unclassified' stamp: 'pt 8/15/2009 08:52'!
originalForm

	^ originalForm
! !

!WorldAdaptingForm methodsFor: 'as yet unclassified' stamp: 'pt 8/15/2009 08:59'!
originalForm: aForm
	"Set the receiver's form"

	originalForm := aForm.
	self layoutChanged.
! !

!WorldAdaptingForm methodsFor: 'as yet unclassified' stamp: 'pt 8/15/2009 08:57'!
rotationStyle

	^ rotationStyle
! !

!WorldAdaptingForm methodsFor: 'as yet unclassified' stamp: 'pt 8/15/2009 08:57'!
rotationStyle: aSymbol
	"Set my rotation style to #normal, #leftRight, #upDown, or #none. Styles mean:
		#normal		-- continuous 360 degree rotation
		#leftRight		-- quantize angle to left or right facing
		#upDown		-- quantize angle to up or down facing
		#none			-- do not rotate"

	rotationStyle := aSymbol.
	self layoutChanged.
! !

!WorldAdaptingForm methodsFor: 'as yet unclassified' stamp: 'pt 8/15/2009 08:58'!
scalePoint

	scalePoint ifNil: [scalePoint := 1.0@1.0].
	^ scalePoint
! !

!WorldAdaptingForm methodsFor: 'as yet unclassified' stamp: 'pt 8/15/2009 08:58'!
scalePoint: aPoint

	scalePoint := aPoint.
	self layoutChanged.
! !

!WorldAdaptingForm methodsFor: 'as yet unclassified' stamp: 'pt 8/15/2009 08:40'!
useInterpolation
	^(self valueOfProperty: #useInterpolation ifAbsent:[false]) 
		and:[Smalltalk includesKey: #B3DRenderEngine]! !

!WorldAdaptingForm methodsFor: 'as yet unclassified' stamp: 'pt 8/15/2009 08:40'!
useInterpolation: aBool
	(aBool == true and: [ Smalltalk includesKey: #B3DRenderEngine ])
		ifTrue:[self setProperty: #useInterpolation toValue: aBool]
		ifFalse:[self removeProperty: #useInterpolation].
	self layoutChanged. "to regenerate the form"
! !
